from abc import abstractmethod
from datetime import datetime
from typing import Protocol

from clifinance.application.dto.balance import Balance
from clifinance.application.dto.expense import ExpenseDTO, ExpenseTypeDTO


class DataGateway(Protocol):
    @abstractmethod
    def get_balance(self) -> Balance:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses(self) -> list[ExpenseDTO]:
        raise NotImplementedError

    @abstractmethod
    def add_expense(self, exp: ExpenseDTO) -> None:
        raise NotImplementedError

    @abstractmethod
    def update_expense(self, expense: ExpenseDTO) -> None:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_type(self, type: ExpenseTypeDTO) -> list[ExpenseDTO]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_date(self, date: datetime) -> list[ExpenseDTO]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_price(self, price: float) -> list[ExpenseDTO]:
        raise NotImplementedError
