from abc import abstractmethod
from datetime import datetime
from typing import Protocol

from clifinance.application.dto.expense import ExpenseDTO, ExpenseTypeDTO


class FileSession(Protocol):
    @abstractmethod
    def commit(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def rollback(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def add(self, expense: ExpenseDTO) -> None:
        raise NotImplementedError

    @abstractmethod
    def update(self, expense: ExpenseDTO) -> None:
        raise NotImplementedError

    @abstractmethod
    def delete(self, expense: ExpenseDTO) -> None:
        raise NotImplementedError

    @abstractmethod
    def get(
        self,
        expense_id: int | None,
        type: ExpenseTypeDTO | None,
        date: datetime | None,
        amount: int | None,
    ) -> list[ExpenseDTO]:
        raise NotImplementedError

    @abstractmethod
    def get_all(self) -> list[ExpenseDTO]:
        raise NotImplementedError
