from dataclasses import dataclass
from datetime import datetime
from enum import Enum


class ExpenseTypeDTO(Enum):
    INCOME = "Income"
    EXPENSE = "Expense"


@dataclass(frozen=True)
class ExpenseDTO:
    id: int | None
    type: ExpenseTypeDTO
    date: datetime
    amount: float
    description: str

    def to_json(self) -> dict:
        return {
            "id": self.id,
            "type": self.type.value,
            "date": self.date.isoformat(),
            "amount": self.amount,
            "description": self.description,
        }

    @staticmethod
    def from_json(data: dict) -> "ExpenseDTO":
        return ExpenseDTO(
            id=data["id"],
            type=(ExpenseTypeDTO.INCOME if data["type"] == "Income" else ExpenseTypeDTO.EXPENSE),
            date=datetime.fromisoformat(data["date"]),
            amount=data["amount"],
            description=data["description"],
        )
