from datetime import datetime

from clifinance.application.abstractions.protocols.gateway import DataGateway
from clifinance.application.abstractions.protocols.session import FileSession
from clifinance.application.dto.balance import Balance
from clifinance.application.dto.expense import ExpenseDTO, ExpenseTypeDTO


class JsonDataGateway(DataGateway):
    def __init__(self, session: FileSession) -> None:
        self.session = session

    def get_balance(self) -> Balance:
        data = self.session.get_all()
        balance = sum(x.amount for x in data)
        income = sum(x.amount for x in data if x.type == ExpenseTypeDTO.INCOME)
        expense = sum(x.amount for x in data if x.type == ExpenseTypeDTO.EXPENSE)
        return Balance(
            balance=round(balance, 2),
            imcome_balance=round(income, 2),
            expense_balance=round(expense, 2),
        )

    def get_all_expenses(self) -> list[ExpenseDTO]:
        return self.session.get_all()

    def add_expense(self, exp: ExpenseDTO) -> None:
        self.session.add(exp)
        self.session.commit()

    def update_expense(self, expense: ExpenseDTO) -> None:
        self.session.update(expense)

    def get_all_expenses_by_type(self, type: ExpenseTypeDTO) -> list[ExpenseDTO]:
        return [x for x in self.session.get_all() if x.type == type]

    def get_all_expenses_by_date(self, date: datetime) -> list[ExpenseDTO]:
        return [x for x in self.session.get_all() if x.date == date]

    def get_all_expenses_by_price(self, price: float) -> list[ExpenseDTO]:
        return [x for x in self.session.get_all() if x.amount == price]
