from datetime import datetime

from clifinance.application.abstractions.protocols.driver import FileDriver
from clifinance.application.abstractions.protocols.session import FileSession
from clifinance.application.dto.expense import ExpenseDTO, ExpenseTypeDTO


class JsonFileSession(FileSession):
    def __init__(self, driver: FileDriver) -> None:
        self.driver = driver
        self.data: list[ExpenseDTO] = driver.read()
        self.__dirty_new: list[ExpenseDTO] = list()
        self.__dirty_updated: list[ExpenseDTO] = list()
        self.__dirty_deleted: list[ExpenseDTO] = list()

    def commit(self) -> None:
        for expense in self.__dirty_new:
            self.data.append(expense)

        for expense in self.__dirty_updated:
            for item in self.data:
                if item.id == expense.id:
                    item = expense
                    break

        for expense in self.__dirty_deleted:
            for item in self.data:
                if item.id == expense.id:
                    del item
                    break
        self.__dirty_new.clear()
        self.__dirty_updated.clear()
        self.__dirty_deleted.clear()

        self.driver.write(self.data)

    def rollback(self) -> None:
        self.data = self.driver.read()

    def add(self, expense: ExpenseDTO) -> None:
        if not self.data:
            last_id = -1
        else:
            last_id = max({x.id for x in self.data if x.id is not None})

        self.__dirty_new.append(
            ExpenseDTO(
                id=last_id + 1,
                type=expense.type,
                date=expense.date,
                amount=expense.amount,
                description=expense.description,
            )
        )

    def update(self, expense: ExpenseDTO) -> None:
        self.__dirty_updated.append(expense)

    def delete(self, expense: ExpenseDTO) -> None:
        self.__dirty_deleted.append(expense)

    def get(
        self,
        expense_id: int | None = None,
        type: ExpenseTypeDTO | None = None,
        date: datetime | None = None,
        amount: int | None = None,
    ) -> list[ExpenseDTO]:
        result: list[ExpenseDTO] = list()
        for expense in self.data:
            if expense_id is not None and expense.id != expense_id:
                continue
            if type is not None and expense.type != type:
                continue
            if date is not None and expense.date != date:
                continue
            if amount is not None and expense.amount != amount:
                continue

            result.append(expense)

        return result

    def get_all(self) -> list[ExpenseDTO]:
        return self.data
